// +-------------------------------- DISCLAIMER ---------------------------------+
// |                                                                             |
// | This application program is provided to you free of charge as an example.   |
// | Despite the considerable efforts of Euresys personnel to create a usable    |
// | example, you should not assume that this program is error-free or suitable  |
// | for any purpose whatsoever.                                                 |
// |                                                                             |
// | EURESYS does not give any representation, warranty or undertaking that this |
// | program is free of any defect or error or suitable for any purpose. EURESYS |
// | shall not be liable, in contract, in torts or otherwise, for any damages,   |
// | loss, costs, expenses or other claims for compensation, including those     |
// | asserted by third parties, arising out of or in connection with the use of  |
// | this program.                                                               |
// |                                                                             |
// +-----------------------------------------------------------------------------+
//
// ImageProcessing.cs: implementation of the ImageProcessing class.
//
//////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

/// Open eVision Namespace
using Euresys.Open_eVision_1_2;


namespace LotMixingLearn
{
    
    class ImageProcessing
    {
        /// Source and temporary images
        private EImageBW8 m_SrcImg = new EImageBW8();

        // EasyOCR object
        private EOCR m_Ocr = new EOCR();

        private int m_n32Index = 0;
        private int m_nRecoClasses = 0;
        private string m_sChar = ""; 

        /// <summary>
        /// Constructor
        /// </summary>
        public ImageProcessing()
        {

        }

        /// <summary>
        /// Destructor
        /// </summary>
        ~ImageProcessing()
        {

        }

        /// <summary>
        /// Initialization
        /// </summary>
        public void Initialize()
        {
            try
            {
                /// Load the image featuring the characters  
                m_SrcImg.Load(@"..\..\..\..\Resources\font\OCR-b.bmp");

                /// Set the characters color
                m_Ocr.TextColor =  EOCRColor.WhiteOnBlack;

                // Build blobs
                m_Ocr.BuildObjects(m_SrcImg);

                // Build chars
                m_Ocr.FindAllChars(m_SrcImg);
          
          }
          catch (EException exc)
          {
              MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
          }

        }

        /// <summary>
        /// Drawing
        /// </summary>
        public void Draw(Graphics g)
        {
            try
            {
                /// Exit if no image has been loaded
                if (m_SrcImg.IsVoid)
                    return;

                /// Draw the source image
                m_SrcImg.Draw(g);

                /// Draw the character Patterns
                m_Ocr.DrawChars(g);
        
            }
            catch (EException exc)
            {
                MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

         public void SaveFontFile(string  _mFilename)
         {
            try
            {
                /// Save the font file
                m_Ocr.Save(_mFilename);
            }
            catch(EException exc)
            {
                MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
         }


        public void OnLButtonDblClk(object sender, MouseEventArgs e)
        {
	        try
	        {
                /// Check if a pattern has been selected 
                if (!(m_Ocr.HitChars(e.X, e.Y, out m_n32Index)))
		        return;

		        CharClassAndCodeForm pDlg = new CharClassAndCodeForm();
                DialogResult result = pDlg.ShowDialog();

		       if(result == DialogResult.Cancel )
		        { 
			        return;
		        }
		        else
		        {
                    /// Get parameters for learning
                    pDlg.GetParameters(out m_sChar, out m_nRecoClasses);
			        LearnPattern();
		        }

                pDlg.Close();
	        }
	        catch(EException exc)
	        {
                MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
	        }

        }

        public void LearnPattern()
        {
	        try
	        {
                /// If a class or code  has not been specified, inform about it
                /// and don't do anything.
                if (m_nRecoClasses == 0 || m_sChar == "")
                {

                    MessageBox.Show("It is mandatory to select a character class and code. \nThe character has not been added to the font. ", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                /// Add the pattern to the font
                m_Ocr.LearnPattern(m_SrcImg, m_n32Index, m_sChar[0], m_nRecoClasses);

 	        }
	        catch(EException exc)
	        {
                MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
	        }

        }

    }
}


